((
	********************************************
	***                                      ***
	*** INTS.FTH                             ***
	***                                      ***
	*** Description:                         ***
	***   Example interrupt code for         ***
	***   RTX2000/1/10 ROM Target.           ***
	***                                      ***
	*** History:                             ***
	***   V1.000 J Lee 12/8/93               ***
	***   Original release.                  ***
	***                                      ***
	********************************************
))


((
****************************************************
****						****
**** 	Interrupt Masking and Re-Enabling	****
****						****
****************************************************

It is left to the programmer of an ISR to re-enable interrupts if required
(RTX disables interrupts on entry and enables on exit).  Also any change to
interrupt masks must be implemented by the programmer if requried.
))


only forth definitions
decimal



((
****************************************************
****						****
**** 		Timer0 Example			****
****						****
****************************************************
))

\
\ Counts for number of timer0 interrupts - using reset timer preload value.
\  Choose one of 8Mhz or 10Mhz 
\

clock-khz 10000 =
if(
  153 constant Second	\ 10Mhz : 153 x 65536 = 10027008 approx. 1 second
)else(
  122 constant Second	\  8Mhz	: 122 x 65536 = 7995392 approx. 1 second
)endif

\
\ count interrupts to give seconds, and count seconds
\
variable #Timer0	\ number of timer0 ints.
variable Counter	\ counter to increment after each Second #timer0's


: Timer0-Example	( -- ; example timer0 routine )
  #Timer0 @ 0=			\ reached 0? (counted 1 second)
  If				\ y:
    1 Counter +!		\   increment number of seconds counter
    Second #Timer0 !		\   reset #timer0 ticker
  Else				\ n:
    #Timer0 decr		\   decrement timer0 interrupt counter
  Endif				\
;

: Init-Timer0-Example	( -- ; initialise timer0 example )
  int-base ibc!					\ set base of vectors
  mask-all					\ mask all interrupts
  assign timer0-example to-do timer0-isr	\ set up the ISR
  0 Counter !					\ no seconds counted yet
  Second #Timer0 !				\ set int. counter for 1sec.
;


((
	Type the following to try the timer0 example:

	DI
	INIT-TIMER0-EXAMPLE
	TIMER0 UNMASK
	EI
	COUNTER ?		\ try this quite a few times!

	TIMER0 MASK		\ do this when bored!
	DI
))



((
****************************************************
****						****
**** 		EI1 Example			****
****						****
****************************************************
))


((
External interrupts are level triggered and require the ISR to clear the 
source of the interrupt.  Otherwise, after finishing the ISR, the RTX will
probably end up in the same ISR because the interrupt will still be present!


: EI1-Example		( -- ; example ei1 routine )
  ...				\ do some code
  clear-ei1-source		\ clear ei1 interrupt source
  ...				\ do some more code
;
))



((
****************************************************
****						****
**** 		NMI Example			****
****						****
****************************************************
))


((
: NMI-Example		( -- ; NMI example routine )
  ...				\ some code
  Begin				\ not safe to return from NMI so:
    saving-grace		\   perform some operation forever
  Again				\
;


It is not safe to return from an NMI on the RTX2000 and RTX2001, so the
NMI would only normally be used to perform last 'minute' essential code
before a fatal event such as power failure.

However, preliminary RTX2010 data indicates that the NMI behaviour can be
altered so that a second mode is accessed by using a previously reserved bit
in the CR register.  This second mode would allow for a safe return from the
NMI.
))


((
****************************************************
****						****
**** 	Software Interrupt Example		****
****						****
****************************************************
))


((
The software interrupt ISR must have a -SOFTINT instruction to clear the swi
flip-flip interrupt source.  This instruction must allow 2 clock cycles
before the EXIT from the ISR.  This is shown in the example but is not really
necessary because additional code to restore the processor registers has to
be executed by the MPE interrupt handler (takes more than 2 clock cycles) and
so is only shown for completeness.
))


: swi-Example		( -- ; swi example routine )
  cr cr ." Software Interrupt"	\ show interrupt has been run
  cr cr				\
  -softint			\ clear swi flip-flop
  noop noop			\ must have delay f 2 clock cycles
;				

: Test-swi		( -- ; test swi )
  cr ." Starting"		\ show underway in foreground
  softint			\ cause swi
  noop noop			\ wait 2 cycles to ensure swi has occurred
  ." Finished"			\ finished
;


((
	To try the example type the following:

	DI  MASK-ALL
	ASSIGN SWI-EXAMPLE TO-DO SWI-ISR
	SWI UNMASK
	EI
	
	TEST-SWI		\ try it a few times

	SWI MASK		\ do this when bored!

	TEST-SWI		\ shouldn't run swi ISR this time

	DI
))



((
****************************************************
****						****
**** 	Phantom Interrupt Example		****
****						****
****************************************************
))


((
The Phantom interrupt or No-Interrupt interrrupt (same one) occurs when
an interrupt is generated by the interrupt controller to the CPU but when
the CPU attempts to process the interrupt, the interrupt source has
disappeared and instead the vector of the phantom interrupt is used.
))


\ count for number of phantom interrupts
\
variable #Phantoms

: Phantom-Example	( -- ; phantom interrupt routine )
  #Phantoms incr		\ increment counter
;


((
	To use it set it up as follows:

	DI
	ASSIGN PHANTOM-EXAMPLE TO-DO PHANTOM-ISR
	EI

Then try and generate one!  Check by doing:

	#PHANTOMS ?

))

	


	
  

